@@ -432,6 +432,8 @@ on_sign_out_handled (GoaIdentityService    *self,
   else
     goa_identity_service_manager_complete_sign_out (GOA_IDENTITY_SERVICE_MANAGER (self),
                                                     invocation);
+
+  g_object_unref (invocation);
 }
 
 static void
@@ -476,7 +478,7 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
                               GSimpleAsyncResult *operation_result)
 {
   GError *error;
-  GoaIdentity *identity;
+  GoaIdentity *identity = NULL;
 
   error = NULL;
   identity = goa_identity_manager_get_identity_finish (manager, result, &error);
@@ -485,7 +487,9 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
     {
       g_debug ("GoaIdentityService: Identity could not be signed out: %s",
                error->message);
-      return;
+      g_simple_async_result_take_error (operation_result, error);
+      g_simple_async_result_complete_in_idle (operation_result);
+      goto out;
     }
 
   g_object_set_data_full (G_OBJECT (operation_result),
@@ -499,7 +503,11 @@ on_got_identity_for_sign_out (GoaIdentityManager *manager,
                                           NULL,
                                           (GAsyncReadyCallback)
                                           on_identity_signed_out,
-                                          operation_result);
+                                          g_object_ref (operation_result));
+
+ out:
+  g_clear_object (&identity);
+  g_object_unref (operation_result);
 }
 
 static gboolean
@@ -548,6 +556,8 @@ on_secret_keys_exchanged (GoaIdentityService *self,
                                                                   invocation,
                                                                   output_key);
     }
+
+  g_object_unref (invocation);
 }
 
 static void
@@ -576,7 +586,6 @@ on_caller_watched (GDBusConnection    *connection,
                                        GCR_ERROR_UNRECOGNIZED,
                                        _("Initial secret key is invalid"));
       g_simple_async_result_complete_in_idle (operation_result);
-      g_object_unref (operation_result);
       return;
     }
 
@@ -657,6 +666,7 @@ goa_identity_service_handle_exchange_secret_keys (GoaIdentityServiceManager *man
                        g_strdup (sender),
                        GUINT_TO_POINTER (watch_id));
 
+  g_object_unref (operation_result);
   return TRUE;
 }
 
@@ -1658,6 +1668,8 @@ ensure_credentials_for_accounts (GoaIdentityService *self)
 
       ensure_account_credentials (self, object);
     }
+
+  g_list_free_full (accounts, g_object_unref);
 }
 
 static void
